@@ -5,6 +5,7 @@ from typing import List, Union
 from openrgb.network import NetworkClient
 # from dataclasses import dataclass
 from time import sleep
+from os import environ
 
 
 class LED(utils.RGBObject):
@@ -18,15 +19,18 @@ class LED(utils.RGBObject):
         self.device_id = device_id
         self.comms = network_client
 
-    def set_color(self, color: utils.RGBColor):
+    def set_color(self, color: utils.RGBColor, fast: bool = False):
         '''
         Sets the color of the LED
 
         :param color: the color to set the LED to
+        :param fast: If you care more about quickly setting colors than having correct internal state data, then set :code:`fast` to :code:`True`
         '''
         self.comms.send_header(self.device_id, utils.PacketType.NET_PACKET_ID_RGBCONTROLLER_UPDATESINGLELED, struct.calcsize("i3bx"))
         buff = struct.pack("i", self.id) + color.pack()
         self.comms.sock.send(buff)
+        if not fast:
+            self.comms.requestDeviceData(self.device_id)
 
 
 class Zone(utils.RGBObject):
@@ -46,13 +50,14 @@ class Zone(utils.RGBObject):
         self.comms = network_client
         self.id = zone_id
 
-    def set_color(self, color: utils.RGBColor, start: int = 0, end: int = 0):
+    def set_color(self, color: utils.RGBColor, start: int = 0, end: int = 0, fast: bool = False):
         '''
         Sets the LEDs color in the zone between start and end
 
         :param color: the color to set the leds to
         :param start: the first LED to change
         :param end: the last LED to change
+        :param fast: If you care more about quickly setting colors than having correct internal state data, then set :code:`fast` to :code:`True`
         '''
         if end == 0:
             end = len(self.leds)
@@ -60,14 +65,17 @@ class Zone(utils.RGBObject):
         buff = struct.pack("IH", self.id, end - start) + (color.pack())*(end - start)
         buff = struct.pack("I", len(buff)) + buff
         self.comms.sock.send(buff)
+        if not fast:
+            self.comms.requestDeviceData(self.device_id)
 
-    def set_colors(self, colors: List[utils.RGBColor], start: int = 0, end: int = 0):
+    def set_colors(self, colors: List[utils.RGBColor], start: int = 0, end: int = 0, fast: bool = False):
         '''
         Sets the LEDs colors in the zone between start and end
 
         :param colors: the list of colors, one per led
         :param start: the first LED to change
         :param end: the last LED to change
+        :param fast: If you care more about quickly setting colors than having correct internal state data, then set :code:`fast` to :code:`True`
         '''
         if end == 0:
             end = len(self.leds)
@@ -77,6 +85,11 @@ class Zone(utils.RGBObject):
         buff = struct.pack("IH", self.id, end - start) + b''.join((color.pack() for color in colors))
         buff = struct.pack("I", len(buff)) + buff
         self.comms.sock.send(buff)
+        if not fast:
+            self.comms.requestDeviceData(self.device_id)
+
+    def update_device_colors(self):
+        pass
 
 
 class Device(utils.RGBObject):
@@ -93,16 +106,18 @@ class Device(utils.RGBObject):
         self.modes = data.modes
         self.colors = data.colors
         self.active_mode = data.active_mode
+        self.data = data
         self.id = device_id
         self.comms = network_client
 
-    def set_color(self, color: utils.RGBColor, start: int = 0, end: int = 0):
+    def set_color(self, color: utils.RGBColor, start: int = 0, end: int = 0, fast: bool = False):
         '''
         Sets the LEDs color between start and end
 
         :param color: the color to set the leds to
         :param start: the first LED to change
         :param end: the last LED to change
+        :param fast: If you care more about quickly setting colors than having correct internal state data, then set :code:`fast` to :code:`True`
         '''
         if end == 0:
             end = len(self.leds)
@@ -114,14 +129,17 @@ class Device(utils.RGBObject):
         buff = struct.pack("H", end - start) + (color.pack())*(end - start)
         buff = struct.pack("I", len(buff)) + buff
         self.comms.sock.send(buff)
+        if not fast:
+            self.comms.requestDeviceData(self.id)
 
-    def set_colors(self, colors: List[utils.RGBColor], start: int = 0, end: int = 0):
+    def set_colors(self, colors: List[utils.RGBColor], start: int = 0, end: int = 0, fast: bool = False):
         '''
         Sets the LEDs colors between start and end
 
         :param colors: the list of colors, one per led
         :param start: the first LED to change
         :param end: the last LED to change
+        :param fast: If you care more about quickly setting colors than having correct internal state data, then set :code:`fast` to :code:`True`
         '''
         if end == 0:
             end = len(self.leds)
@@ -135,6 +153,8 @@ class Device(utils.RGBObject):
         buff = struct.pack("H", end - start) + b''.join((color.pack() for color in colors))
         buff = struct.pack("I", len(buff)) + buff
         self.comms.sock.send(buff)
+        if not fast:
+            self.comms.requestDeviceData(self.id)
 
     def set_mode(self, mode: Union[int, str, utils.ModeData]):
         '''
@@ -148,14 +168,19 @@ class Device(utils.RGBObject):
             mode = self.modes[mode]
         elif type(mode) == str:
             mode = next((m for m in self.modes if m.name.lower() == mode.lower()))
-        # print(mode)
-        size, data = mode.pack()
+        data = mode.pack()
         self.comms.send_header(
             self.id,
             utils.PacketType.NET_PACKET_ID_RGBCONTROLLER_UPDATEMODE,
-            size
+            len(data)
         )
         self.comms.sock.send(data)
+        self.active_mode = mode.id
+        self.data.active_mode = self.active_mode
+        if len(mode.colors) == len(self.colors):
+            self.colors = mode.colors
+            self.data.colors = self.colors
+        self.data.modes = self.modes
 
     def set_custom_mode(self):
         self.comms.send_header(
@@ -165,14 +190,20 @@ class Device(utils.RGBObject):
         )
 
 
-class OpenRGBClient(object):
+class OpenRGBClient(utils.RGBObject):
     '''
     This is the only class you should ever need to instantiate.  It initializes
     the communication, gets the device information, sets the devices to the
     custom mode and creates Devices, Zones, and LEDs for you.
     '''
 
-    def __init__(self, address: str = "127.0.0.1", port: int = 1337, name: str = "openrgb-python"):
+    def __init__(self, address: str = "127.0.0.1", port: int = 6742, name: str = "openrgb-python", custom: bool = True):
+        '''
+        :param address: the ip address of the SDK server
+        :param port: the port of the SDK server
+        :param name: the string that will be displayed on the OpenRGB SDK tab's list of clients
+        :param custom: whether or not to set all your devices to custom control mode on initializtion
+        '''
         self.comms = NetworkClient(self._callback, address, port, name)
         self.address = address
         self.port = port
@@ -181,11 +212,12 @@ class OpenRGBClient(object):
         while self.device_num == 0:
             sleep(.2)
         self.devices = [None for x in range(self.device_num)]
-        for x in range(self.device_num):
-            self.comms.requestDeviceData(x)
-        sleep(1)  # Giving the client time to recieve the device data
-        for dev in self.devices:
-            dev.set_custom_mode()
+        self.get_device_info()
+        while any((dev is None for dev in self.devices)):
+            sleep(.2)
+        if custom:
+            for dev in self.devices:
+                dev.set_custom_mode()
 
     def __repr__(self):
         return f"OpenRGBClient(address={self.address}, port={self.port}, name={self.name})"
@@ -197,28 +229,17 @@ class OpenRGBClient(object):
             if self.devices[device] is None:
                 self.devices[device] = Device(data, device, self.comms)
             else:
-                self.devices[device].data = data
+                self.devices[device].__init__(data, device, self.comms)
 
-    def set_color(self, color: utils.RGBColor):
+    def set_color(self, color: utils.RGBColor, fast: bool = False):
         '''
         Sets the color of every device.
 
         :param color: the color to set the devices to
+        :param fast: If you care more about quickly setting colors than having correct internal state data, then set :code:`fast` to :code:`True`
         '''
         for device in self.devices:
-            device.set_color(color)
-
-    def clear(self):
-        '''
-        Turns all of the LEDs off
-        '''
-        self.set_color(utils.RGBColor(0, 0, 0))
-
-    def off(self):
-        '''
-        See OpenRGBClient.clear
-        '''
-        self.clear()
+            device.set_color(color, fast=fast)
 
     def get_devices_by_type(self, type: utils.DeviceType) -> List[Device]:
         '''
@@ -227,3 +248,54 @@ class OpenRGBClient(object):
         :param type: what type of device you want to get
         '''
         return [device for device in self.devices if device.type == type]
+
+    def load_profile(self, name: str, directory: str = ''):
+        '''
+        Loads an OpenRGB profile file
+
+        :param name: the name of the profile
+        :param directory: what directory the profile is in.  Defaults to HOME/.config/OpenRGB
+        '''
+        if directory == '':
+            directory = environ['HOME'].rstrip("/") + "/.config/OpenRGB"
+        with open(f'{directory}/{name}.orp', 'rb') as f:
+            controllers = utils.Profile.unpack(f).controllers
+            pairs = []
+            for device in self.devices:
+                for new_controller in controllers:
+                    if new_controller.name == device.name \
+                            and new_controller.device_type == device.type \
+                            and new_controller.metadata.description == device.metadata.description:
+                        controllers.remove(new_controller)
+                        pairs.append((new_controller, device))
+            # print("Pairs:")
+            for new_controller, device in pairs:
+                # print(device.name, new_controller.name)
+                if new_controller.colors != device.colors:
+                    device.set_colors(new_controller.colors)
+                # print(new_controller.active_mode)
+                if new_controller.active_mode != device.active_mode:
+                    device.set_mode(new_controller.active_mode)
+
+    def save_profile(self, name: str, directory: str = ''):
+        '''
+        Saves the current state of all of your devices to an OpenRGB profile
+        file
+
+        :param name: the name of the profile to save
+        :param directory: what directory to save the profile in.  Defaults to HOME/.config/OpenRGB
+        '''
+        self.get_device_info()
+        if directory == '':
+            directory = environ['HOME'].rstrip("/") + "/.config/OpenRGB"
+        with open(f'{directory.rstrip("/")}/{name}.orp', 'wb') as f:
+            f.write(utils.Profile(self.devices).pack())
+
+    def get_device_info(self):
+        '''
+        Gets the current state of your devices from the SDK server.  Useful if
+        you change something from the gui or another SDK client and need to
+        sync up the changes.
+        '''
+        for x in range(self.device_num):
+            self.comms.requestDeviceData(x)
